#ifndef FrameCPP_VERSION_4_FrEndOfFile_HH
#define FrameCPP_VERSION_4_FrEndOfFile_HH

#include "general/types.hh"

#include "framecpp/Version4/FrameSpec.hh"

#include "framecpp/Common/FrEndOfFile.hh"

namespace FrameCPP
{
  namespace Version_4
  {
    class FrEndOfFile
      : public Common::FrEndOfFile
    {
    public:
      typedef INT_4U nFrames_type;
      typedef INT_4U nBytes_type;
      typedef INT_4U chkFlag_type;
      typedef INT_4U chkSum_type;
      typedef INT_4U seekTOC_type;

      FrEndOfFile( );
      virtual ~FrEndOfFile( );

      static const char* const StructName( );

      static const Common::Description* StructDescription( );

      virtual Common::FrameSpec::size_type
      Bytes( const Common::StreamBase& Stream ) const;

      virtual FrEndOfFile* Clone( ) const;

      virtual FrEndOfFile* Create( Common::IStream& Stream ) const;

      virtual const char* const ObjectStructName( ) const;

      virtual void Write( Common::OStream& Stream ) const;

      /// \brief comparison operator
      bool operator==( const FrEndOfFile& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

      static const Common::FrameSpec::Info::frame_object_types s_object_id
      = Common::FrameSpec::Info::FSI_FR_END_OF_FILE;

      FrEndOfFile::seekTOC_type GetSeekTOC( ) const;

      virtual FrEndOfFile::seekTOC_cmn_type SeekTOC( ) const;

      virtual void VerifyObject( Common::Verify& Verifier,
				 Common::IFrameStream& Stream ) const;

    protected:
      nFrames_type	m_nFrames;
      nBytes_type	m_nBytes;
      chkFlag_type	m_chkFlag;
      chkSum_type	m_chkSum;
      seekTOC_type	m_seekTOC;

      /// \brief Down grade an object
      virtual demote_ret_type
      demote( INT_2U Target,
	      demote_arg_type Obj,
	      Common::IStream* Stream ) const;

      /// \brief Upgrade an object
      virtual promote_ret_type
      promote( INT_2U Target,
	       promote_arg_type Obj,
	       Common::IStream* Stream ) const;

    private:
      using Common::FrameSpec::Object::Create;

      FrEndOfFile( Common::IStream& Source );

      chkSum_cmn_type Filter( const Common::IStream& Stream,
			      Common::CheckSum& Filt,
			      void* Buffer,
			      INT_8U Size ) const;

    }; // class - FrEndOfFile

    inline const char* const FrEndOfFile::
    StructName( )
    {
      static const CHAR* class_name( "FrEndOfFile" );
      return class_name;
    }

    inline FrEndOfFile::seekTOC_type FrEndOfFile::
    GetSeekTOC( ) const
    {
      return m_seekTOC;
    }


    inline Common::FrEndOfFile::seekTOC_cmn_type FrEndOfFile::
    SeekTOC( ) const
    {
      return GetSeekTOC( );
    }
  } // namespace - Version_4
} // namespace - FrameCPP

#endif /* FrameCPP_VERSION_4_FrEndOfFile_HH */
